#!/usr/bin/env python
"""
:Author Patrik Valkovic
:Created 10.08.2017 20:32
:Licence GNUv3
Part of grammpy-transforms

"""

from .remove_nongenerating_nonterminals import remove_nongenerating_nonterminals